==> flutter_mylibrary-0.4.49.tar.gz: lib/base/provider/request/view_state_refresh_list_model.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/base/base_view_state.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/base/provider/view_state_model.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';


/// 基于
abstract class ViewStateRefreshListModel<T> extends ViewStateModel {

  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  onCompleted(List<T> data) {}



  ///获取数据
  Future<List<T>> _getData({bool clearData, bool isRefreshCall = true}) {
    RequestInfo requestInfo = getRequestInfo();
    getOtherData();
    return EasyDio().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onError: (e){
          handleError(e, isRefreshCall);
        },
    ).then((res){
      try{
        List<T> resData = [];

        if (isNotEmpty(res.data)) {
          resData = res.data is List?List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
        }


        if (clearData) {
          list.clear();
        }

        if (isNotEmpty(resData)) {
          list.addAll(resData);
        }

        onCompleted(resData);

        if (isEmpty(list)) {
          ///空数据
          setError(
            errorType: ViewStateErrorType.noDataError
          );
        }else{
          setIdle();
        }

        refreshController.resetLoadState();
        if (isRefreshCall) {
          refreshController.finishRefresh(success: true);
        }else{
          refreshController.finishLoad(success: true, noMore: !hasMore);
        }
        return resData;
      }catch(e){
        handleError(e, isRefreshCall);
        return null;
      }
    });
  }


  void handleError(e, bool isRefreshCall){
    if (hasMore) {
      _currentPageNum--;
    }
    refreshController.resetLoadState();
    if (isRefreshCall) {
      refreshController.finishRefresh(success: false);
    }else{
      refreshController.finishLoad(success: false, noMore: !hasMore);
    }

    if (!isInit) {
      setError(
        e: e
      );
    }
  }


  void initData({bool retry = false}){
    if (retry) {
      setBusy();
    }
    refresh();
  }



  ///获取一个公共的刷新控件(EasyRefresh)
  Widget getEasyRefreshWidget({ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder, ViewStateModelWidgetBuilder<List<T>> builder, Widget emptyWidget, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(viewStateModel: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget, showBusyWidget: showBusyWidget);
    return EasyRefresh.custom(
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      emptyWidget: emptyErrorWidget,
      slivers: isNotEmpty(widgetListBuilder)? widgetListBuilder(list) : [
        isEmpty(emptyErrorWidget)? isNotEmpty(builder)? builder(list) : SliverToBoxAdapter(child: Container(),) :SliverToBoxAdapter(child: Container(),),
      ],
      onRefresh: isBusy? null : () async{
        await refresh();
      },
      onLoad: (isBusy || !hasMore)? null : () async{
        await loadMore();
      },
      controller: refreshController,
    );
  }





  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  /// 下拉刷新
  ///
  /// [init] 是否是第一次加载
  /// true:  Error时,需要跳转页面
  /// false: Error时,不需要跳转页面,直接给出提示
  Future<List<T>> refresh({bool clearData = true}) async {
    _currentPageNum = firstPage();
    var data = await _getData(clearData: clearData, isRefreshCall: true);
    return data;
  }

  /// 上拉加载更多
  Future<List<T>> loadMore() async {
    _currentPageNum = ++_currentPageNum;
    var data = await _getData(clearData: false, isRefreshCall: false);
    return data;
  }

  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }


  bool get hasMore => getRequestInfo().useLoadMore?totalNum > list.length? true : false : false;


}

==> flutter_mylibrary-0.4.61.tar.gz: lib/base/provider/request/view_state_refresh_list_model.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/base/base_view_state.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/base/provider/view_state_model.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';


/// 基于
abstract class ViewStateRefreshListModel<T> extends ViewStateModel {

  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  onCompleted(List<T> data) {}



  ///获取数据
  Future<List<T>> _getData({bool clearData, bool isRefreshCall = true}) async{
    requestInfo = await getRequestInfo();
    getOtherData();
    return EasyDio().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onError: (e){
          handleError(e, isRefreshCall);
        },
    ).then((res){
      try{
        List<T> resData = [];

        if (isNotEmpty(res.data)) {
          resData = res.data is List?List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
        }


        if (clearData) {
          list.clear();
        }

        if (isNotEmpty(resData)) {
          list.addAll(resData);
        }

        onCompleted(resData);

        if (isEmpty(list)) {
          ///空数据
          setError(
            errorType: ViewStateErrorType.noDataError
          );
        }else{
          setIdle();
        }

        refreshController.resetLoadState();
        if (isRefreshCall) {
          refreshController.finishRefresh(success: true);
        }else{
          refreshController.finishLoad(success: true, noMore: !hasMore);
        }
        return resData;
      }catch(e){
        handleError(e, isRefreshCall);
        return null;
      }
    });
  }


  void handleError(e, bool isRefreshCall){
    if (hasMore) {
      _currentPageNum--;
    }
    refreshController.resetLoadState();
    if (isRefreshCall) {
      refreshController.finishRefresh(success: false);
    }else{
      refreshController.finishLoad(success: false, noMore: !hasMore);
    }

    if (!isInit) {
      setError(
        e: e
      );
    }
  }


  void initData({bool retry = false}){
    if (retry) {
      setBusy();
    }
    refresh();
  }



  ///获取一个公共的刷新控件(EasyRefresh)
  Widget getEasyRefreshWidget({ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder, ViewStateModelWidgetBuilder<List<T>> builder, Widget emptyWidget, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(viewStateModel: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget, showBusyWidget: showBusyWidget);
    return EasyRefresh.custom(
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      emptyWidget: emptyErrorWidget,
      slivers: isNotEmpty(widgetListBuilder)? widgetListBuilder(list) : [
        isEmpty(emptyErrorWidget)? isNotEmpty(builder)? builder(list) : SliverToBoxAdapter(child: Container(),) :SliverToBoxAdapter(child: Container(),),
      ],
      onRefresh: isBusy? null : () async{
        await refresh();
      },
      onLoad: (isBusy || !hasMore)? null : () async{
        await loadMore();
      },
      controller: refreshController,
    );
  }





  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  /// 下拉刷新
  ///
  /// [init] 是否是第一次加载
  /// true:  Error时,需要跳转页面
  /// false: Error时,不需要跳转页面,直接给出提示
  Future<List<T>> refresh({bool clearData = true}) async {
    _currentPageNum = firstPage();
    var data = await _getData(clearData: clearData, isRefreshCall: true);
    return data;
  }

  /// 上拉加载更多
  Future<List<T>> loadMore() async {
    _currentPageNum = ++_currentPageNum;
    var data = await _getData(clearData: false, isRefreshCall: false);
    return data;
  }

  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }


  bool get hasMore => requestInfo.useLoadMore?totalNum > list.length? true : false : false;


}
